require 'dsx/dml'
require 'dsx/writer'
require 'dsx/operation'
require 'securerandom'

def get_card_number
  rand(10000000000..99999999999)
end

def get_name
  'test_' + SecureRandom::hex(4)
end

def test_credentials
  {address: '127.0.0.1', user: 'teset' , password: 'testing_password'}
end

describe Dsx::Dml do
  describe "Creating a changset string" do
    
    name = get_name()
    number = get_card_number()
    zone = Dsx::Dml::Zone.new(130, 2)
    
    changeset = zone.use(number)
    
    changeset.table('Names').write({
        FName: name, 
        LName: name + "_L",
        Company: 'Bio5',
        Visitor: '',
        Trace: '',
        Notes: ''
    }).table('UDF').write({
        UdfNum: 2,
        UdfText: number
    }).table('Cards').write({
        Code: number,
        PIN: 1234,
        StartDate: '09/15/2013',
        StopDate: '04/11/2023',
        NumUses: 9999
    })
    
    changeset_string = changeset.to_string
    
    it "Should use DML compatible newlines" do
      changeset_string.should include("\r")
      changeset_string.should_not include("\n")
    end
    
    it "Should have some properly formatted fields and values" do
      changeset_string.should include("^#{name}^^^")
      changeset_string.should include("^#{number}^^^")
      changeset_string.should include("\rF UdfNum ^2^^^\r")
    end
    
    # writer = Dsx::Dml::Writer.new(test_credentials)
    # writer.net_sftp(changeset)
    
  end
  
  describe "Using an operation" do
    name = get_name()
    op = Dsx::Dml::Operation.new(location_num:  130, 
                                 udf_num:       2, 
                                 first_name:    name, 
                                 last_name:     name + '_L', 
                                 company:       'Bio5', 
                                 identifier:    get_card_number())
    op.access_levels += ['BIO5 Test Level 1', 'BIO5 Test Level 2']
    op.access_levels -= ['Master Access Level']
    op.linking_level = 1

    op_string = op.to_s
    
    it "Should use DML compatible newlines" do
      op_string.should include("\r")
      op_string.should_not include("\n")
    end
    
    it "Should have access level commands" do
      op_string.should include("\rF AddAcl ^BIO5 Test Level 1^^^\r")
      op_string.should include("\rF AddAcl ^BIO5 Test Level 2^^^\r")
      op_string.should include("\rF DelAcl ^Master Access Level^^^\r")
      op_string.should include("\rF Loc ^130^^^\r")
      op_string.should include("\rF OLL ^1^^^\r")
    end
      
    # writer = Dsx::Dml::Writer.new(test_credentials)
    # writer.curl(op.changeset)
    
  end
end
